#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

GE<-function(fT,or_T,fE,or_E,fM,no_LD,LD,LD_measure,P0,Kp,alpha,no_int,int_TE,test,power,N,mode_inh,r_path,data_path,outputfile,hypothesis,par_spec){
    #OR estimation. 
    ORME_path<-paste(r_path,"ORME.R",sep="")
    source(ORME_path)
    
    #Estimation of the baseline risk of disease in unexposed individuals
    P1_marginalE_path<-paste(r_path,"P1_marginalE.R",sep="")
    source(P1_marginalE_path)
    
    #Estimation of marginal OR's of the environmental exposure
    marginalE_path<-paste(r_path,"marginalE.R",sep="")
    source(marginalE_path)
    
    #Estimation of the marginal OR's at the marker locus
    marginalG_path<-paste(r_path,"marginalG.R",sep="")
    source(marginalG_path)
    
    #Estimation of power/sample size required to detect the interaction/joint effect (depending on the hypothesis). 
    Power_file<-paste(r_path,"Power_GE.R",sep="")
    source(Power_file)
    
    #Estimation of power/sample size. 
    Power_file_marginalE<-paste(r_path,"Power_marginalE.R",sep="")
    source(Power_file_marginalE)
    
    #Estimation of power/sample size. 
    Power_file_mainG<-paste(r_path,"Power_mainG.R",sep="")
    source(Power_file_mainG)
    
    
    nrow<-no_LD*no_int
    Output_return<-matrix(NA,nrow,7)
    int_count<-1
        
    if(power==FALSE){
        result_name<-"Power"
        result_name_g<-"PowerGeneMarginal"
        result_name_e<-"PowerEnvMarginal"
        if(hypothesis=="GE"){result_name_ge<-"PowerInteraction"
        }else{result_name_ge<-"PowerJointTest"}
    }else{
        result_name<-"SampleSize"
        result_name_g<-"SampleSizeGeneMarginal"
        result_name_e<-"SampleSizeEnvMarginal"
        if(hypothesis=="GE"){result_name_ge<-"SampleSizeInteraction"
        }else{result_name_ge<-"SampleSizeJointTest"}
    }
    
        result_g<-matrix(NA,no_int,no_LD)
        result_e<-matrix(NA,no_int,no_LD)
        result_ge<-matrix(NA,no_int,no_LD)
        for(interaction_TE in 1:no_int){
            if(length(Kp)>1){
                Kp_interaction<-Kp[interaction_TE]
                P0_interaction<-P0
            }else if(length(P0)>1){
                P0_interaction<-P0[interaction_TE]
                Kp_interaction<-Kp
            }else if(length(P0)==1&&length(Kp)==1){
                P0_interaction<-P0
                Kp_interaction<-Kp
            }
            
            #OR estimation. 
            Output<-ORME(fT,or_T,fE,or_E,fM,no_LD,LD,LD_measure,P0_interaction,Kp_interaction,int_TE[interaction_TE],mode_inh,r_path) 
            
            or_M<-Output[,9]
            or_ME<-Output[,10]
            int_ME<-or_ME/(or_M*or_E)
            alphaP1_marginalG<-log(Output[,11]/(1-Output[,11]))
            alphaP1_int<-log(Output[,12]/(1-Output[,12]))
            P1_or_E_bar<-P1_marginalE(or_E,fE,P0_interaction,Kp_interaction)
            alphaP1_marginalE<-log(P1_or_E_bar/(1-P1_or_E_bar))
            
            #Estimation of the environmental and genetic marginal effects. 
            or_E_bar<-marginalE(or_M,or_E,int_ME,fM,no_LD,alphaP1_marginalE,mode_inh)
            or_M_bar<-marginalG(or_M,or_E,int_ME,fE,no_LD,alphaP1_marginalG)
            
            #Result file for OR's. 
            P0.x<-round(P0_interaction,digits=5)
            Kp.x<-round(Kp_interaction,digits=5)
            P0_vec<-rep(P0.x,no_LD)
            Kp_vec<-rep(Kp.x,no_LD)
            P1_or_E_bar.x<-round(P1_or_E_bar,digits=5)
            P1_E_vec<-rep(P1_or_E_bar.x,no_LD)
            Output_round<-round(Output,digits=5)
            output_new<-matrix(NA,no_LD,15)
            output_new[,1:10]<-Output_round[,1:10]
            output_new[,11]<-P1_E_vec
            output_new[,12]<-round(or_E_bar,digits=5)
            output_new[,13]<-round(or_M_bar,digits=5)
            output_new[,c(14,15)]<-Output_round[,c(11,12)]
            result_or<-cbind(output_new,P0_vec,Kp_vec)
            P0_round<-round(P0_interaction,digits=2)
            Kp_round<-round(Kp_interaction,digits=2)
            if(hypothesis=="GE"){
                if(par_spec=="P0"){
                    if(int_TE[interaction_TE]==1){
                        file_name<-paste(data_path,"GE_noInteraction_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_P0_",P0_round,".out",sep="")
                    }else{
                        file_name<-paste(data_path,"Interaction_int_TE_",int_TE[interaction_TE],"_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_P0_",P0_round,".out",sep="")
                    }
                }else{
                    if(int_TE[interaction_TE]==1){
                        file_name<-paste(data_path,"GE_noInteraction_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_Kp_",Kp_round,".out",sep="")
                    }else{
                        file_name<-paste(data_path,"Interaction_int_TE_",int_TE[interaction_TE],"_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_Kp_",Kp_round,".out",sep="")
                    }
                }
            }else{
                if(par_spec=="P0"){
                    if(int_TE[interaction_TE]==1){
                        file_name<-paste(data_path,"GGE_noInteraction_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_P0_",P0_round,".out",sep="")
                    }else{
                        file_name<-paste(data_path,"JointTest_int_TE_",int_TE[interaction_TE],"_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_P0_",P0_round,".out",sep="")
                    }
                }else{
                    if(int_TE[interaction_TE]==1){
                        file_name<-paste(data_path,"GGE_noInteraction_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_Kp_",Kp_round,".out",sep="")
                    }else{
                        file_name<-paste(data_path,"JointTest_int_TE_",int_TE[interaction_TE],"_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_Kp_",Kp_round,".out",sep="")
                    }
                }
            }
            write.table(result_or, file=file_name,row.names=FALSE,col.names=c("fE","fT","fM","D","r2","or_E","or_T","or_TE","or_M","or_ME","or_E_bar","or_M_bar","P1_marginalE","P1_marginalG","P1_int","P0","Kp")) 
            
            d_TM<-round(Output[,4],digits=5)
            r2<-round(Output[,5],digits=5)
            
            #Sample size/power estimation. 
            result_e[interaction_TE,]<-Power_marginalE(fE,or_E_bar,alphaP1_marginalE,alpha,test,power,N,r_path,no_LD)
            result_g[interaction_TE,]<-Power_mainG(fM,or_M_bar,alphaP1_marginalG,alpha,test,power,N,mode_inh,r_path,no_LD)
            if(any(int_ME!=1)){
                result_ge[interaction_TE,]<-Power_GE(fE,or_E,fM,or_M,int_ME,alphaP1_int,alpha,test,power,N,mode_inh,r_path,no_LD,hypothesis)    
            }else{
                if(N==FALSE){
                    result_ge[interaction_TE]<-Inf
                }else{
                    result_ge[interaction_TE]<-0
                }
            }
            #Result file for power/sample size. 
            fM_rep<-rep(fM,no_LD)
            int_rep<-rep(int_TE[interaction_TE],no_LD)
            Output_return[c(int_count:(no_LD+int_count-1)),]<-cbind(fM_rep, int_rep, d_TM, r2, result_e[interaction_TE,], result_g[interaction_TE,], result_ge[interaction_TE,])
            colNames<-c("fM","int_TE","D","r2",result_name_e,result_name_g,result_name_ge)
            xx<-matrix(Output_return[c(int_count:(no_LD+int_count-1)),],nrow=no_LD,ncol=7)
            if(hypothesis=="GE"){
                if(par_spec=="P0"){
                    result_file<-paste(outputfile,result_name,"_interaction_int_TE_",int_TE[interaction_TE],"_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_P0_",P0_round,"_alpha_",alpha,"_test_",test,"_power_",power,"_N_",N,"_mode_inh_",mode_inh,".out",sep="")
                }else{
                    result_file<-paste(outputfile,result_name,"_interaction_int_TE_",int_TE[interaction_TE],"_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_Kp_",Kp_round,"_alpha_",alpha,"_test_",test,"_power_",power,"_N_",N,"_mode_inh_",mode_inh,".out",sep="")
                }
            }else{
                if(par_spec=="P0"){
                    result_file<-paste(outputfile,result_name,"_JointTest_int_TE_",int_TE[interaction_TE],"_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_P0_",P0_round,"_alpha_",alpha,"_test_",test,"_power_",power,"_N_",N,"_mode_inh_",mode_inh,".out",sep="")
                }else{
                    result_file<-paste(outputfile,result_name,"_JointTest_int_TE_",int_TE[interaction_TE],"_fM_",fM,"_fT_",fT,"_fE_",fE,"_or_T_",or_T,"_or_E_",or_E,"_Kp_",Kp_round,"_alpha_",alpha,"_test_",test,"_power_",power,"_N_",N,"_mode_inh_",mode_inh,".out",sep="")
                }
            }
            write.table(xx,file=result_file,row.names=FALSE,col.names=colNames)
            int_count<-int_count+no_LD
        }
    
    return(Output_return)
   
}
