#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

ORM<-function(fT,or_T,fM,no_LD,LD,LD_measure,P0,Kp,mode_inh,r_path){
            
    #Estimation of a range of LD values which fit to fT and fM
    LD_path<-paste(r_path,"LD.R",sep="")
    source(LD_path)
    
    #Estimation of the baseline risk of disease in individuals who do not carry the marker allele. 
    P1_mainG_path<-paste(r_path,"P1_mainG.R",sep="")
    source(P1_mainG_path)
    
    ft<-1-fT
    fm<-1-fM
    
    #Estimation of r2/D. 
    if(LD_measure=="D"){
        d_TM<-LD
        r2<-(d_TM^2)/(fM*fm*fT*ft)
    }else if(LD_measure=="r2"){
        r2<-LD
        d_TM<-sqrt(r2*fT*ft*fM*fm)
    }
    
    #Joint distribution for disease and marker locus in the population, depending on the linkage disequilibrium paramter between alleles T and M. 
    fTM<-d_TM+(fT*fM)
    ftm<-d_TM+(ft*fm)
    fTm<--d_TM+(fT*fm)
    ftM<--d_TM+(ft*fM)
    
    if(any(fTM>min(fT+10^(-10),fM+10^(-10)))){
        control_ld<-LD(fT,fM)
        if(LD_measure=="r2"){
            r2_min<-control_ld[3]
            r2_max<-control_ld[4]
            stop("Invalid LD value (r2=",r2,") given these marker and disease allele frequencies (fM=",fM,", fT=",fT,")!\n r2 should be in the range ",r2_min," to ",r2_max,"! ")
        }else{
            d_TM_min<-control_ld[1]
            d_TM_max<-control_ld[2]
            stop("Invalid LD value (D=",d_TM,") given these marker and disease allele frequencies (fM=",fM,", fT=",fT,")!\n D should be in the range ",d_TM_min," to ",d_TM_max,"! ")
        }
    }

    #Non-normalized joint frequencies of exposure, disease locus, and marker locus among cases. 
    gtm<-ftm
    gtM<-ftM
    
    gTm<-or_T*fTm
    gTM<-or_T*fTM
        
    sum_g<-gtm+gTM+gTm+gtM
    
    #Normalized joint frequencies of exposure, disease locus, and marker locus among cases
    gtm<-gtm/sum_g
    gTM<-gTM/sum_g
    gTm<-gTm/sum_g
    gtM<-gtM/sum_g
    
    #Estimation of the OR at marker locus (formula by Zondervan & Cardon (Nature Reviews, 2004, 5:89-100) and Ackerman et al. (Genome Biology, 2003, 4,4,R24)). 
    or_M<-1+(d_TM*(or_T-1))/(((fT*fm-d_TM)*(or_T-1)+fm)*fM)
    
    gt<-gtm+gtM
    gT<-gTm+gTM
    gm<-gtm+gTm
    
    #Estimation of P1, the baseline risk of disease in the non-marker-allele carriers. 
    probD_m<-P1_mainG(or_M,fM,no_LD,P0,Kp,mode_inh)
    
    output<-matrix(NA, no_LD, 7)
    output[, 1]<-fT
    output[, 2]<-fM
    output[, 3]<-d_TM
    output[, 4]<-r2
    output[, 5]<-or_T
    output[, 6]<-or_M
    output[, 7]<-probD_m
    
    return(output)
}
