#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Estimation of the baseline risk of disease in unexposed individuals using a Newton-type algorithm (nlm()). 
#The function to be minimized (fct()) originates from equation (3) in Gauderman, Statistics in Medicine, 2002, 21:35-50. 
P1_marginalE<-function(or_E,fE,P0,probD){
    probD_e<-0
    alphaP0<-log(P0/(1-P0))
    beta<-log(or_E)
    fe<-1-fE
    
    fct<-function(alphaP1,beta,fE,fe,probD) abs(((exp(alphaP1+beta)/(1+exp(alphaP1+beta))*fE+exp(alphaP1)/(1+exp(alphaP1))*fe)-probD))
    loglik<-nlm(fct, p=alphaP0,beta=beta,fE=fE,fe=fe,probD=probD)
    alpha_estimate<-loglik$estimate
    
    probD_e<-exp(alpha_estimate)/(1+exp(alpha_estimate))
    return(probD_e)
}
