#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Estimation of the power/sample size required to detect the gene-environment interaction effect(s). 
Power_GE<-function(fE,or_E,fM,or_M,int_ME,alphaP1,alpha,test,power,N,mode_inh,r_path,no_LD,hypothesis){
                 
    step1<-paste(r_path,"ML_GE.R",sep="")
    source(step1)

    #Estimation of the true model parameters of the logistic model. 
    beta<-matrix(NA,no_LD,3)
    beta[,1]<-log(or_M)
    beta[,2]<-log(or_E)
    beta[,3]<-log(int_ME)
    
    #Step 1: Maximization of the expected log-likelihoods L1 and L0 and calculation of lambda. 
    lambda<-ML_int_hypothesis(fM,fE,mode_inh,alphaP1,beta,no_LD,hypothesis)
    
    if(hypothesis=="GE"){
        #Step 2: Calculation of za, zb, i.e. the (1-a)th and (1-b)th percentile of the standard normal distribution (a=alpha, b=1-power). 
        if(test==1){q_alpha<-qnorm((1-alpha))
        }else if(test==2){q_alpha<-qnorm((1-(alpha/2)))
        }
        
        #Step 3: Power/sample Size estimation. 
        if(N==FALSE){
            q_beta<-qnorm(power)
            N<-rep(NA,no_LD)
            N<-round(((q_alpha+q_beta)^2)/lambda)
            return(N)
        }else if(power==FALSE){
            if(test==1){
                power<-round(pnorm(sqrt(N*lambda)-q_alpha,mean=0,sd=1,lower.tail=TRUE,log.p = FALSE),digits=4)
            }else if(test==2){
                power<-round(pnorm(sqrt(N*lambda)-q_alpha,mean=0,sd=1,lower.tail=TRUE,log.p = FALSE)+pnorm(-sqrt(N*lambda)-q_alpha,mean=0,sd=1,lower.tail=TRUE, log.p = FALSE),digits=4)
            }
            return(power)
        }
    }else{
        #Step 2: Calculation of the 1-alpha quantile of the chi-square distribution
        q_alpha<-qchisq((1-alpha),df=2,ncp=0,lower.tail=TRUE,log.p = FALSE)
        #Step 3: Power estimation. 
        non_centr_par<-lambda*N
        cdf_chisq<-pchisq(q_alpha,df=2,ncp=non_centr_par,lower.tail=TRUE,log.p=FALSE)
        power<-round(1-cdf_chisq,digits=4)
        return(power)
    }
}
